import _JSON$stringify from 'babel-runtime/core-js/json/stringify';
import _Promise from 'babel-runtime/core-js/promise';
import _getIterator from 'babel-runtime/core-js/get-iterator';
import _Object$keys from 'babel-runtime/core-js/object/keys';
import _slicedToArray from 'babel-runtime/helpers/slicedToArray';
import _Object$assign from 'babel-runtime/core-js/object/assign';
import _classCallCheck from 'babel-runtime/helpers/classCallCheck';
import _createClass from 'babel-runtime/helpers/createClass';
import * as ko from 'knockout';
import { isBool, isEmpty, isNumber, isUndefined, entries, omit } from './utils';

var Query = function () {
    function Query(config, group) {
        _classCallCheck(this, Query);

        this._group = group;
        if (isUndefined(Query._raw[this._group])) {
            Query._raw[this._group] = {};
            Query._refs[this._group] = 1;
        } else {
            Query._refs[this._group]++;
        }
        this.set(config);
    }

    _createClass(Query, [{
        key: 'set',
        value: function set(config) {
            var _this = this;

            var defaults = _Object$assign({}, Query.getDefaults(config));
            var group = this._group;
            var fromQS = Query.fromQS(group);
            entries(config).forEach(function (_ref) {
                var _ref2 = _slicedToArray(_ref, 2),
                    name = _ref2[0],
                    _ref2$ = _ref2[1],
                    paramConfig = _ref2$ === undefined ? {} : _ref2$;

                _this[name] = Query._raw[group][name];
                if (isUndefined(_this[name])) {
                    var _default = defaults[name];
                    var coerce = paramConfig.coerce || function (x) {
                        return x;
                    };
                    var init = !isUndefined(fromQS[name]) ? fromQS[name] : paramConfig.initial;
                    _this[name] = Query._raw[group][name] = Query.createQueryParam(group, name, _default, init, coerce);
                } else {
                    _this[name].set(paramConfig);
                }
            });
            ko.tasks.runEarly();
        }
    }, {
        key: 'toJS',
        value: function toJS() {
            return omit(ko.toJS(Query._raw[this._group]), isUndefined);
        }
    }, {
        key: 'toString',
        value: function toString() {
            return Query.stringify(this.toJS());
        }
    }, {
        key: 'asObservable',
        value: function asObservable() {
            var _this2 = this;

            return ko.pureComputed(function () {
                return _this2.toJS();
            });
        }
    }, {
        key: 'clear',
        value: function clear() {
            var _this3 = this;

            _Object$keys(Query._raw[this._group]).forEach(function (k) {
                return Query._raw[_this3._group][k].clear();
            });
        }
    }, {
        key: 'dispose',
        value: function dispose() {
            var group = this._group;
            if (--Query._refs[group] === 0) {
                var current = _Object$assign({}, Query.fromQS(), Query.getCleanQuery());
                delete current[group];
                Query.writeQueryString(current);
                delete Query._raw[group];
            }
        }
    }], [{
        key: 'parse',
        value: function parse(str) {
            return Query._parser.parse(str);
        }
    }, {
        key: 'stringify',
        value: function stringify(obj) {
            return Query._parser.stringify(obj);
        }
    }, {
        key: 'create',
        value: function create(config, group) {
            return new Query(config, group);
        }
    }, {
        key: 'setParser',
        value: function setParser(parser) {
            Query._parser = parser;
        }
    }, {
        key: 'getQueryString',
        value: function getQueryString() {
            var matches = /\?([^#]*)/.exec(location.search + location.hash);
            return matches ? matches[1] : '';
        }
    }, {
        key: 'fromQS',
        value: function fromQS(group) {
            var query = this.parse(this.getQueryString());
            return (isUndefined(group) ? query : query[group]) || {};
        }
    }, {
        key: 'getCleanQuery',
        value: function getCleanQuery() {
            var _query = {};
            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                for (var _iterator = _getIterator(entries(Query._raw)), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    var _ref3 = _step.value;

                    var _ref4 = _slicedToArray(_ref3, 2);

                    var g = _ref4[0];
                    var q = _ref4[1];

                    _query[g] = ko.toJS(omit(q, function (v) {
                        return v.isDefault() || isUndefined(v()) || isEmpty(v()) && !isNumber(v()) && !isBool(v());
                    }));
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }

            var group = undefined;
            if (_query[group]) {
                _Object$assign(_query, _query[group]);
                delete _query[group];
            }
            return _query;
        }
    }, {
        key: 'writeQueryString',
        value: function writeQueryString(_query) {
            if (!_query) {
                _query = this.getCleanQuery();
            }
            var qs = Query.stringify(_query);
            var currentUrl = location.pathname + location.search + location.hash;
            var hashbang = currentUrl.indexOf('#!') > -1;
            var newUrl = void 0;
            if (hashbang) {
                newUrl = currentUrl.replace(/(?:\?[^#]+|$)/, qs ? '?' + qs : '');
            } else {
                var pathnameMatches = /([^?#]*)/.exec(currentUrl);
                if (!pathnameMatches) {
                    throw new Error('Failed to write query string; could not parse current url');
                }
                var currentPathname = pathnameMatches[1];
                var hashMatches = /(#[^!]*)/.exec(currentUrl);
                newUrl = currentPathname;
                if (qs) {
                    newUrl += '?' + qs;
                }
                if (hashMatches) {
                    newUrl += hashMatches[1];
                }
            }
            history.replaceState(history.state, document.title, newUrl);
        }
    }, {
        key: 'queueQueryStringWrite',
        value: function queueQueryStringWrite() {
            var _this4 = this;

            if (!this._queuedUpdate) {
                this._queuedUpdate = new _Promise(function (resolve) {
                    ko.tasks.schedule(function () {
                        Query.writeQueryString();
                        resolve();
                        _this4._queuedUpdate = false;
                    });
                });
            }
            return this._queuedUpdate;
        }
    }, {
        key: 'createQueryParam',
        value: function createQueryParam(group, name, __default, // tslint:disable-line variable-name
        init, coerce) {
            var _this5 = this;

            var _default = ko.observable(ko.toJS(__default));
            var _p = ko.observable(isUndefined(init) ? _default() : init);
            var isDefault = ko.pureComputed(function () {
                return p() === _default();
            });
            var p = ko.pureComputed({
                read: function read() {
                    return _p();
                },
                write: function write(v) {
                    if (isUndefined(v)) {
                        v = _default();
                    }
                    if (coerce) {
                        v = coerce(v);
                    }
                    _p(v);
                    Query.queueQueryStringWrite()
                    // tslint:disable-next-line no-console
                    .catch(function (err) {
                        return console.error('[@profiscience/knockout-contrib-querystring] error queueing write');
                    });
                }
            });
            _Object$assign(p, {
                isDefault: isDefault,
                set: function set(d) {
                    if (!_this5.isParamConfigObject(d)) {
                        d = d;
                        if (isDefault() || isUndefined(p())) {
                            p(d);
                        }
                        _default(d);
                    } else {
                        d = d;
                        if (d.coerce) {
                            coerce = d.coerce;
                        }
                        if (isDefault() || isUndefined(p()) || !isUndefined(d.initial)) {
                            p(isUndefined(d.initial) ? d.default : d.initial);
                        }
                        if (d.default) {
                            _default(d.default);
                        }
                    }
                },
                clear: function clear() {
                    return p(_default());
                }
            });
            return p;
        }
    }, {
        key: 'isParamConfigObject',
        value: function isParamConfigObject(c) {
            return c && (c.default || c.initial || c.coerce);
        }
    }, {
        key: 'getDefaults',
        value: function getDefaults(config) {
            var _this6 = this;

            var defaults = {};
            entries(config).forEach(function (_ref5) {
                var _ref6 = _slicedToArray(_ref5, 2),
                    k = _ref6[0],
                    v = _ref6[1];

                return defaults[k] = _this6.isParamConfigObject(v) ? v.default : v;
            });
            return defaults;
        }
    }]);

    return Query;
}();

export default Query;

Query._raw = {};
Query._refs = {};
Query._parser = {
    parse: function parse(str) {
        return JSON.parse(decodeURIComponent(str || '{}'));
    },
    stringify: function stringify(obj) {
        return _JSON$stringify(obj) === '{}' ? '' : encodeURIComponent(_JSON$stringify(obj));
    }
};
//# sourceMappingURL=index.js.map