'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});

var _knockout = require('knockout');

var ko = _interopRequireWildcard(_knockout);

var _utils = require('./utils');

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

class Query {
    constructor(config, group) {
        this._group = group;
        if ((0, _utils.isUndefined)(Query._raw[this._group])) {
            Query._raw[this._group] = {};
            Query._refs[this._group] = 1;
        } else {
            Query._refs[this._group]++;
        }
        this.set(config);
    }
    set(config) {
        const defaults = Object.assign({}, Query.getDefaults(config));
        const group = this._group;
        const fromQS = Query.fromQS(group);
        (0, _utils.entries)(config).forEach(([name, paramConfig = {}]) => {
            this[name] = Query._raw[group][name];
            if ((0, _utils.isUndefined)(this[name])) {
                const _default = defaults[name];
                const coerce = paramConfig.coerce || (x => x);
                const init = !(0, _utils.isUndefined)(fromQS[name]) ? fromQS[name] : paramConfig.initial;
                this[name] = Query._raw[group][name] = Query.createQueryParam(group, name, _default, init, coerce);
            } else {
                this[name].set(paramConfig);
            }
        });
        ko.tasks.runEarly();
    }
    toJS() {
        return (0, _utils.omit)(ko.toJS(Query._raw[this._group]), _utils.isUndefined);
    }
    toString() {
        return Query.stringify(this.toJS());
    }
    asObservable() {
        return ko.pureComputed(() => this.toJS());
    }
    clear() {
        Object.keys(Query._raw[this._group]).forEach(k => Query._raw[this._group][k].clear());
    }
    dispose() {
        const group = this._group;
        if (--Query._refs[group] === 0) {
            const current = Object.assign({}, Query.fromQS(), Query.getCleanQuery());
            delete current[group];
            Query.writeQueryString(current);
            delete Query._raw[group];
        }
    }
    static parse(str) {
        return Query._parser.parse(str);
    }
    static stringify(obj) {
        return Query._parser.stringify(obj);
    }
    static create(config, group) {
        return new Query(config, group);
    }
    static setParser(parser) {
        Query._parser = parser;
    }
    static getQueryString() {
        const matches = /\?([^#]*)/.exec(location.search + location.hash);
        return matches ? matches[1] : '';
    }
    static fromQS(group) {
        const query = this.parse(this.getQueryString());
        return ((0, _utils.isUndefined)(group) ? query : query[group]) || {};
    }
    static getCleanQuery() {
        const _query = {};
        for (const [g, q] of (0, _utils.entries)(Query._raw)) {
            _query[g] = ko.toJS((0, _utils.omit)(q, v => v.isDefault() || (0, _utils.isUndefined)(v()) || (0, _utils.isEmpty)(v()) && !(0, _utils.isNumber)(v()) && !(0, _utils.isBool)(v())));
        }
        const group = undefined;
        if (_query[group]) {
            Object.assign(_query, _query[group]);
            delete _query[group];
        }
        return _query;
    }
    static writeQueryString(_query) {
        if (!_query) {
            _query = this.getCleanQuery();
        }
        const qs = Query.stringify(_query);
        const currentUrl = location.pathname + location.search + location.hash;
        const hashbang = currentUrl.indexOf('#!') > -1;
        let newUrl;
        if (hashbang) {
            newUrl = currentUrl.replace(/(?:\?[^#]+|$)/, qs ? '?' + qs : '');
        } else {
            const pathnameMatches = /([^?#]*)/.exec(currentUrl);
            if (!pathnameMatches) {
                throw new Error('Failed to write query string; could not parse current url');
            }
            const currentPathname = pathnameMatches[1];
            const hashMatches = /(#[^!]*)/.exec(currentUrl);
            newUrl = currentPathname;
            if (qs) {
                newUrl += '?' + qs;
            }
            if (hashMatches) {
                newUrl += hashMatches[1];
            }
        }
        history.replaceState(history.state, document.title, newUrl);
    }
    static queueQueryStringWrite() {
        if (!this._queuedUpdate) {
            this._queuedUpdate = new Promise(resolve => {
                ko.tasks.schedule(() => {
                    Query.writeQueryString();
                    resolve();
                    this._queuedUpdate = false;
                });
            });
        }
        return this._queuedUpdate;
    }
    static createQueryParam(group, name, __default, // tslint:disable-line variable-name
    init, coerce) {
        const _default = ko.observable(ko.toJS(__default));
        const _p = ko.observable((0, _utils.isUndefined)(init) ? _default() : init);
        const isDefault = ko.pureComputed(() => p() === _default());
        const p = ko.pureComputed({
            read() {
                return _p();
            },
            write(v) {
                if ((0, _utils.isUndefined)(v)) {
                    v = _default();
                }
                if (coerce) {
                    v = coerce(v);
                }
                _p(v);
                Query.queueQueryStringWrite()
                // tslint:disable-next-line no-console
                .catch(err => console.error('[@profiscience/knockout-contrib-querystring] error queueing write'));
            }
        });
        Object.assign(p, {
            isDefault,
            set: d => {
                if (!this.isParamConfigObject(d)) {
                    d = d;
                    if (isDefault() || (0, _utils.isUndefined)(p())) {
                        p(d);
                    }
                    _default(d);
                } else {
                    d = d;
                    if (d.coerce) {
                        coerce = d.coerce;
                    }
                    if (isDefault() || (0, _utils.isUndefined)(p()) || !(0, _utils.isUndefined)(d.initial)) {
                        p((0, _utils.isUndefined)(d.initial) ? d.default : d.initial);
                    }
                    if (d.default) {
                        _default(d.default);
                    }
                }
            },
            clear: () => p(_default())
        });
        return p;
    }
    static isParamConfigObject(c) {
        return c && (c.default || c.initial || c.coerce);
    }
    static getDefaults(config) {
        const defaults = {};
        (0, _utils.entries)(config).forEach(([k, v]) => defaults[k] = this.isParamConfigObject(v) ? v.default : v);
        return defaults;
    }
}
exports.default = Query;
Query._raw = {};
Query._refs = {};
Query._parser = {
    parse: str => JSON.parse(decodeURIComponent(str || '{}')),
    stringify: obj => JSON.stringify(obj) === '{}' ? '' : encodeURIComponent(JSON.stringify(obj))
};
//# sourceMappingURL=index.js.map